import argparse
import logging
import os
import subprocess

from fasteners import interprocess_locked
from pa_pylibs.config.configuration import Configuration
from pa_pylibs.logging import logger as pa_logger

from backup_now import resources

LOCK_FILE_PATH = os.path.expanduser('~/.backup_now/lock_files/backup_now.lock')
HOME_DIRECTORY_PATH = os.path.expanduser('~')

CONFIG = Configuration(
    os.path.join(resources.__path__[0], 'backup_now.conf'),
    '~/.backup_now/backup_now.conf'
)


def get_args():
    """
    Parses the command line for arguments.
    :return: The arguments passed to the command line.
    :rtype: argparse.Namespace
    """

    parser = argparse.ArgumentParser('Wrapper for scripting duplicity.')
    parser.add_argument(
        '--offsite',
        help='Run a backup to the registered offsite location.',
        action='store_true'
    )

    parser.add_argument(
        '--onsite',
        help='Run a backup to the registered onsite location.',
        action='store_true'
    )

    parser.add_argument(
        '--notification-list',
        nargs='*',
        help='One or more email addresses to notify with failures.'
    )

    parser.add_argument('--log-console', help='Log to the console instead of file.', action='store_true')

    return parser.parse_args()


@interprocess_locked(LOCK_FILE_PATH)
def backup_onsite():
    """
    Backs up your home directory to the registered onsite location.
    """

    logger = logging.getLogger(__name__)
    logger.debug('Starting onsite backup.')

    result = subprocess.run(
        ['duplicity', HOME_DIRECTORY_PATH, 'file://{}'.format(CONFIG.get_str('nas_share', 'path'))],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        universal_newlines=True
    )

    try:
        result.check_returncode()
    except subprocess.CalledProcessError as e:
        logger.error('Onsite backup failed: {}'.format(e))
    finally:
        logger.debug(result.stdout)

    logger.debug('Finished onsite backup.')


@interprocess_locked(LOCK_FILE_PATH)
def backup_offsite():
    """
    Backs up your home directory to the registered offsite location.
    """

    logger = logging.getLogger(__name__)
    logger.debug('Starting offsite backup.')

    result = subprocess.run(
        [
            'duplicity',
            '--allow-source-mismatch',
            HOME_DIRECTORY_PATH,
            'b2://{}:{}@{}/{}'.format(
                CONFIG.get_str('b2', 'account_id'),
                CONFIG.get_str('b2', 'application_key'),
                CONFIG.get_str('b2', 'bucket_name'),
                CONFIG.get_str('b2', 'path')
            ),
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        universal_newlines=True
    )

    try:
        result.check_returncode()
    except subprocess.CalledProcessError as e:
        logger.error('Offsite backup failed: {}'.format(e))
    finally:
        logger.debug(result.stdout)

    logger.debug('Finished offsite backup.')


def source_env():
    logger = logging.getLogger()

    passphrase_path = CONFIG.get_str('encryption', 'passphrase_path')

    with open(passphrase_path, 'r') as ef:
        logger.debug("Reading passphrase from: {}".format(passphrase_path))
        os.environ['PASSPHRASE'] = ef.read()


def main():
    args = get_args()
    logger = pa_logger.configure(
        application_name='backup_now',
        log_to_console=args.log_console,
        email_errors=True,
        notification_list=args.notification_list,
        smtp_hostport=CONFIG.get_str('email', 'smtp_hostport'),
        username=CONFIG.get_str('email', 'username'),
        password=CONFIG.get_str('email', 'password'),
        from_address=CONFIG.get_str('email', 'from_address'),
        subject='Backup Failed'
    )

    logger.debug('backup_now called with args {}'.format(args))

    source_env()

    if not args.onsite and not args.offsite:
        # Backup all locations
        backup_onsite()
        backup_offsite()
    else:
        if args.onsite:
            backup_onsite()

        if args.offsite:
            backup_offsite()


if __name__ == '__main__':
    main()
